#!/bin/sh



##########################################################################
#                                                                        #
# Version:      1.1                                                      #
#                                                                        #
# Description:  This script calls the eLC-installer's executable file,   #
#               forwarding the given command-line options, and retrieves #
#               the installer's exit code via exchange file, then        #
#               returns that exit code to the caller of this script.     #
#                                                                        #
# Author:       d.stegemann@steinberg.de                                 #
#                                                                        #
# Copyright 2013 Steinberg Media Technologies GmbH                       #
#                                                                        #
##########################################################################



#######################################################
# function define_script_error_codes {}               #
#                                                     #
# Define error codes specific to this script.         #
#######################################################
function define_script_error_codes {
    ERROR_CODE_CANNOT_EXECUTE_INSTALLER=51
    ERROR_CODE_CANNOT_CREATE_EXIT_CODE_EXCHANGE_FILE=52
    ERROR_CODE_CANNOT_RETRIEVE_EXIT_CODE=53
} # define_script_error_codes
#######################################################



#######################################################
# function exit_with_exitcode {}                      #
#                                                     #
# Deletes the exit code exchange file.                #
#######################################################
function exit_with_exitcode {
    # Cleanup: try to delete the exchange file.
    if [ ! -z "$EXIT_CODE_EXCHANGE_PATH" ]; then
        if [ -f "$EXIT_CODE_EXCHANGE_PATH" ]; then
            rm -f "$EXIT_CODE_EXCHANGE_PATH"
        fi
    fi

    exit $1
} # exit_with_exitcode
#######################################################



#######################################################
# function define_paths {}                            #
#                                                     #
# Define/verify the paths to be used in this script.  #
#                                                     #
# Exit in case of failure.                            #
#######################################################
function define_paths {
    # Determine path to installer executable.
    PATH_TO_INSTALLER_EXECUTABLE=`dirname "$0"`/installbuilder.sh

    if [ ! -f "$PATH_TO_INSTALLER_EXECUTABLE" ]; then
        # Installer path doesn't exist; exit now.
        echo " " 
        echo "ERROR: Installer doesn't exist at path '$PATH_TO_INSTALLER_EXECUTABLE'; exit now ($ERROR_CODE_CANNOT_EXECUTE_INSTALLER)."
        exit_with_exitcode $ERROR_CODE_CANNOT_EXECUTE_INSTALLER
    fi

    #EXIT_CODE_EXCHANGE_PATH="/some/path with spaces"
    #EXIT_CODE_EXCHANGE_PATH="/tmp/path/w/out/spaces"
    #return


    # Determine system's temporary folder path.
    TEMP_FOLDER_PATH="/tmp/"

    if [ ! -d "$TEMP_FOLDER_PATH" ]; then
        # Temporary folder path doesn't exist; testing fall-back path.
        if [ ! -z "$TMPDIR" ]; then
            # The 'TMPDIR' variable is not empty.
            TEMP_FOLDER_PATH="$TMPDIR/"
        fi
    fi

    if [ ! -d "$TEMP_FOLDER_PATH" ]; then
        # None of our preferred temporary folder paths exists; exit now.
        echo " " 
        echo "ERROR: Failed to determine temporary folder path; exit now ($ERROR_CODE_CANNOT_CREATE_EXIT_CODE_EXCHANGE_FILE)."
        exit_with_exitcode $ERROR_CODE_CANNOT_CREATE_EXIT_CODE_EXCHANGE_FILE
    fi
} # define_paths
#######################################################



#######################################################
# function create_exitcode_exchange_file {}           #
#                                                     #
# Create the exit code exchange file.                 #
#                                                     #
# Exit in case of failure.                            #
#######################################################
function create_exitcode_exchange_file {
    # Create unique temporary file for exchanging the installer's exit code.
    EXIT_CODE_EXCHANGE_PATH=`mktemp $TEMP_FOLDER_PATH/eLicenserControl_exitcode_exchange_file.tmp.XXXXXX`

    if [ $? -ne 0 ]; then
        # 'mktemp' command failed, so we cannot proceed; exit now.
        echo " " 
        echo "ERROR: Failed to create exit code exchange file; exit now ($ERROR_CODE_CANNOT_CREATE_EXIT_CODE_EXCHANGE_FILE)."
        exit_with_exitcode $ERROR_CODE_CANNOT_CREATE_EXIT_CODE_EXCHANGE_FILE
    fi
} # create_exitcode_exchange_file
#######################################################



#######################################################
# function execute_installer {}                       #
#                                                     #
# Run the installer.                                  #
#                                                     #
# Exit in case of failure.                            #
#######################################################
function execute_installer {
    # Call installer with appropriate options.
    INSTALLER_OPTIONS="$@ --writeexitcodetofile $EXIT_CODE_EXCHANGE_PATH"

    "$PATH_TO_INSTALLER_EXECUTABLE" $INSTALLER_OPTIONS
    EXIT_CODE=$?

    if [ $EXIT_CODE -ne 0 ]; then
        # Installer failed; exit now.
        echo " " 
        echo "ERROR: Failed to call installer at '$PATH_TO_INSTALLER_EXECUTABLE' ($EXIT_CODE); exit now ($ERROR_CODE_CANNOT_EXECUTE_INSTALLER)."
        exit_with_exitcode $ERROR_CODE_CANNOT_EXECUTE_INSTALLER
    fi
} # execute_installer
#######################################################



#######################################################
# function retrieve_and_return_installer_exit_code {} #
#                                                     #
# Retrieve the installer's exit code from the         #
# exit code exchange file, and return it to the       # 
# caller of this script.                              #
#                                                     #
# Exit in case of failure.                            #
#######################################################
function retrieve_and_return_installer_exit_code {
    # Retrieve exit code from exchange file.
    INSTALLER_EXIT_CODE=`more "$EXIT_CODE_EXCHANGE_PATH"`
    CMD_RETURN_VALUE=$?

    if [ ! -s "$EXIT_CODE_EXCHANGE_PATH" -o $CMD_RETURN_VALUE -ne 0 ]; then
        # The exchange file doesn't exist, is empty, or the the 'more' command failed, 
        # so we cannot retrieve any exit code; exit now.
        echo " " 
        echo "ERROR: Failed to retrieve exit code from exchange file '$EXIT_CODE_EXCHANGE_PATH'; exit now ($ERROR_CODE_CANNOT_RETRIEVE_EXIT_CODE)."
        exit_with_exitcode $ERROR_CODE_CANNOT_RETRIEVE_EXIT_CODE
    fi
    
    
    # Return installer exit code to the caller of this script.
    exit_with_exitcode $INSTALLER_EXIT_CODE
} # retrieve_and_return_installer_exit_code
#######################################################



#############
# Main code #
#############

define_script_error_codes
define_paths
create_exitcode_exchange_file
execute_installer $@
retrieve_and_return_installer_exit_code

